# -*- coding: utf-8 -*-
import sys, os, pkginfo, datetime

pkg_info = pkginfo.Develop(os.path.join(os.path.dirname(__file__),'..'))

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.intersphinx'
    ]

intersphinx_mapping = {'http://docs.python.org/dev': None}

# General
source_suffix = '.txt'
master_doc = 'index'
project = pkg_info.name

# copyright
first_year = 2010
current_year = datetime.datetime.now().year
if first_year==current_year:
    date_str = str(current_year)
else:
    date_str = '%s-%s'%(first_year,current_year)
copyright = date_str+' Simplistix Ltd' 


version = release = pkg_info.version
exclude_trees = ['_build']
unused_docs = ['description']
pygments_style = 'sphinx'

# Options for HTML output
html_theme = 'default'
htmlhelp_basename = project+'doc'

# Options for LaTeX output
latex_documents = [
  ('index',project+'.tex', project+u' Documentation',
   'Simplistix Ltd', 'manual'),
]




# Copyright (c) 2010 Simplistix Ltd
# See license.txt for license details.

import os
import pkg_resources

from datetime import datetime
from zc.buildout import easy_install
from zc.buildout.easy_install import Installer
from zc.buildout.easy_install import logger
from zc.buildout.easy_install import IncompatibleVersionError

required_by = {}
picked_versions = {}

# code to patch in
def _log_requirement(ws, req):
    for dist in sorted(ws):
        if req in dist.requires():
            req_ = str(req)
            if req_ not in required_by:
                required_by[req_] = set()
            required_by[req_].add(str(dist.as_requirement()))

original_get_dist = Installer._get_dist
def _get_dist(self, requirement, ws, always_unzip):
    dists = original_get_dist(self, requirement, ws, always_unzip)
    for dist in dists:
        if not (dist.precedence == pkg_resources.DEVELOP_DIST or \
                  (len(requirement.specs) == 1 and \
                       requirement.specs[0][0] == '==')):
            picked_versions[dist.project_name] = dist.version
    return dists

def _constrain(self, requirement):
    version = self._versions.get(requirement.project_name.lower())
    if version:
        if version not in requirement:
            logger.error("The version, %s, is not consistent with the "
                         "requirement, %r.", version, str(requirement))
            raise IncompatibleVersionError("Bad version", version)

        requirement = pkg_resources.Requirement.parse(
            "%s[%s] ==%s" % (requirement.project_name,
                           ','.join(requirement.extras),
                           version))

    return requirement

file_name = None

def start(buildout):
    global file_name

    # normalise version case
    current = Installer._versions
    Installer._versions = {}
    for key,value in current.items():
        Installer._versions[key.lower()]=value

    # patch methods
    easy_install._log_requirement = _log_requirement
    Installer._get_dist = _get_dist
    Installer._constrain = _constrain

    # record file name, if needed
    if 'buildout_versions_file' in buildout['buildout']:
        file_name = buildout['buildout']['buildout_versions_file']
    
def finish(buildout):
    # now check that buildout-versions is constrained!
    version_section = buildout['buildout'].get('versions')
    if not (version_section and
            'buildout-versions' in buildout[version_section]):
        picked_versions['buildout-versions']=pkg_resources.require(
            'buildout-versions'
            )[0].version

    if picked_versions:
        output = ['[versions]']
        required_output = []
        for dist_, version in sorted(picked_versions.items()):
            if dist_ in required_by:
                required_output.append('')
                required_output.append('# Required by:')
                for req_ in sorted(required_by[dist_]):
                    required_output.append('# '+req_)
                target = required_output
            else:
                target = output
            target.append("%s = %s" % (dist_, version))

        output.extend(required_output)
        
        print "Versions had to be automatically picked."
        print "The following part definition lists the versions picked:"
        print '\n'.join(output)
        if file_name:
            if os.path.exists(file_name):
                output[:1] = [
                    '',
                    '# Added by Buildout Versions at %s' % datetime.now(),
                    ]
            output.append('')
            f = open(file_name,'a')
            f.write('\n'.join(output))
            f.close()
            print "This information has been written to %r" % file_name



# Copyright (c) 2010 Simplistix Ltd
# See license.txt for license details.
from __future__ import with_statement

from buildout_versions import _constrain
from pkg_resources import parse_requirements, Requirement
from testfixtures import LogCapture,ShouldRaise,compare
from unittest import TestSuite,makeSuite,TestCase
from zc.buildout.easy_install import IncompatibleVersionError

class FakeInstaller:
    def __init__(self,versions):
        self._versions = versions
        
class TestConstrain(TestCase):

    def setUp(self):
        self._versions = {}
        self.i=FakeInstaller(self._versions)
        self.logging = LogCapture('zc.buildout.easy_install')

    def tearDown(self):
        self.logging.uninstall()

    def _check(self,requirement,expected):
        result = _constrain(
            self.i,
            tuple(parse_requirements(requirement))[0]
            )
        self.failUnless(isinstance(result,Requirement))
        compare(expected,str(result))
        
    def test_normal(self):
        self._versions['a-package']='1.0'
        self._check('a_package','a-package==1.0')
        self.logging.check()
    
    def test_extras(self):
        self._versions['a-package']='1.0'
        self._check('a_package[some,thing]',
                    'a-package[some,thing]==1.0')
        self.logging.check()
    
    def test_capitalisation(self):
        self._versions['a-package']='1.0'
        self._check('A-Package','A-Package==1.0')
        self.logging.check()

    def test_no_version(self):
        self._check('a_package','a-package')
        self.logging.check()
    
    def test_incompatible_version(self):
        self._versions['a-package']='1.0'
        with ShouldRaise(IncompatibleVersionError(
                'Bad version','1.0'
                )):
            self._check('a-package==2.0','xxx')
        self.logging.check(
            ('zc.buildout.easy_install',
             'ERROR',
             "The version, 1.0, is not consistent with"
             " the requirement, 'a-package==2.0'.")
            )

def test_suite():
    return TestSuite((
        makeSuite(TestConstrain),
        ))




# Copyright (c) 2010 Simplistix Ltd
#
# This Software is released under the MIT License:
# http://www.opensource.org/licenses/mit-license.html
# See license.txt for more details.

from setuptools import setup, find_packages

import os

base_dir = os.path.join(os.path.dirname(__file__))

setup(
    name='buildout-versions',
    version='1.5',
    author='Chris Withers',
    author_email='chris@simplistix.co.uk',
    license='MIT',
    description="zc.buildout extension to display and record versions of packages used.",
    long_description=open(os.path.join(base_dir,'docs','description.txt')).read(),
    url='http://packages.python.org/buildout-versions',
    classifiers=[
    'Development Status :: 5 - Production/Stable',
    'License :: OSI Approved :: MIT License',
    ],
    packages = find_packages('src'),
    package_dir = {'':'src'},
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        'zc.buildout',
    ],
    entry_points = { 
        'zc.buildout.extension': [
             'default = buildout_versions:start',
             ],
        'zc.buildout.unloadextension': [
             'default = buildout_versions:finish',
             ],
        },
    extras_require=dict(
           test=[
            'manuel',
            'testfixtures',
            'zc.recipe.egg',
            ],
           )
    )



# Copyright (c) 2010 Simplistix Ltd
#
# See license.txt for more details.

from datetime import datetime
from doctest import REPORT_NDIFF,ELLIPSIS
from manuel import doctest,codeblock,capture
from manuel.testing import TestSuite
from os.path import dirname,join,pardir
from pkg_resources import require
from testfixtures import compare, TempDirectory

import buildout_versions
import os
import re
import zc.buildout.testing

class Fixer:

    def __init__(self,*filters):
        self.filters = filters
    def __call__(self,in_):
        out_ = []
        for line in in_.split('\n'):
            for f in self.filters:
                line = f(line)
                if line is None:
                    break
            if line is None:
                continue
            elif isinstance(line,list):
                out_.extend(line)
            else:
                out_.append(line)
        return '\n'.join(out_)

def setUp(test):
    zc.buildout.testing.buildoutSetUp(test)

    # set up any required eggs
    zc.buildout.testing.install('zc.recipe.egg',test)

    # get some useful bits
    g = test.globs
    write = g['write']
    system = g['system']
    buildout = g['buildout']
    buildout_dir = TempDirectory(path=os.getcwd())

    # set up some packages
    buildout_dir.write('sample_package/setup.py','''
from setuptools import setup
setup(name='sample_package')
''')

    buildout_dir.write('package_a/setup.py','''
from setuptools import setup
setup(name='package_a',version='1.0',install_requires='package_b')
''')
    system(buildout+' setup package_a bdist_egg')
    
    buildout_dir.write('package_b/setup.py','''
from setuptools import setup
setup(name='package_b',version='2.0')
''')
    system(buildout+' setup package_b bdist_egg')

    buildout_dir.write('PackageC/setup.py','''
from setuptools import setup
setup(name='PackageC',version='3.0',install_requires='packaged')
''')
    system(buildout+' setup PackageC bdist_egg')

    buildout_dir.write('PackageD/setup.py','''
from setuptools import setup
setup(name='PackageD',version='4.0')
''')
    system(buildout+' setup PackageD bdist_egg')
    buildout_dir.write('PackageD/setup.py','''
from setuptools import setup
setup(name='PackageD',version='5.0')
''')
    system(buildout+' setup PackageD bdist_egg')

    def buildout_line(line):
        # ignore the $ bin/buildout line
        if not line.startswith('$ bin/buildout'):
            return line

    def couldnt_find(line):
        # ignore spurious "Couldn't find " lines
        if not line.startswith("Couldn't find index page"):
            return line

    spec_re = re.compile('([a-z\.\-]+) = \d')
    def normalise_versions(line):
        # normalise versions to those currently installed
        line = line.replace('\r','')
        match = spec_re.match(line)
        if match:
            package = match.group(1)
            # don't try and fix versions for our test packages
            if not package.lower().startswith('package'):
                version = require(package)[0].version
                line = '%s = %s' % (package,version)
        return line

    def our_index(line):
        if '<our index>' in line:
            line = []
            line.append('find-links =')
            for name in (
                'package_a','package_b',
                'PackageC','PackageD',
                ):
                line.append(
                    '  '+test.globs['start_server'](
                        join(buildout_dir.path,name,'dist')
                             )
                    )
        return line

    def splat_buildout_dir(line):
        return line.replace(
            "'.../sample_package'",
            repr(os.path.join(buildout_dir.path,'sample_package'))
            )

    time_re = re.compile('\d{4}-\d{2}-\d{2} [\d:.]+')
    def fix_time(line):
        return time_re.sub(str(datetime(
                   2010, 7, 30, 15, 44, 45, 23559
                   )),line)

    fix_buildout = Fixer(
        normalise_versions,
        fix_time,
        our_index,
        )

    fix_expected = Fixer(
        buildout_line,
        normalise_versions,
        splat_buildout_dir,
        )

    fix_actual = Fixer(
        couldnt_find,
        normalise_versions,
        )
    
    # the buildout runner and checkers
    def run_buildout(buildout_cfg,expected):
        # setup buildout_versions as a development egg
        write('buildout.cfg','''
[buildout]
develop = %s
parts =
''' % join(dirname(buildout_versions.__file__),pardir,pardir))
        system(buildout)
        # run the buildout we were given
        write('buildout.cfg',fix_buildout(buildout_cfg))
        actual = system(buildout)
        compare(fix_expected(expected),fix_actual(actual))
        
    g['run_buildout'] = run_buildout
    g['buildout_dir'] = buildout_dir
    g['fix_buildout'] = fix_buildout

def test_suite():
    m = capture.Manuel()
    m += codeblock.Manuel()
    return TestSuite(
        m,
        join(dirname(__file__),pardir,pardir,pardir,'docs','use.txt'),
        setUp=setUp,
        tearDown=zc.buildout.testing.buildoutTearDown,
        )
